@@ -4,10 +4,9 @@
 #ifndef __htmlstreamh__
   #include "htmlstream.h"
 #endif
-#ifndef __htmlh__
-  #include "html.h"
-#endif
+
 #include <string>
+#include <iostream>
 
 class DBaseBase {
 public:
