@@ -1,12 +1,10 @@
 #ifndef __dbmgrh__
 #define __dbmgrh__
 
-#ifndef __htmlstreamh__
-  #include "htmlstream.h"
-#endif
 #ifndef __dbaseh__
   #include "dbase.h"
 #endif
+
 #include <map>
 #include <string>
 
