@@ -1,12 +1,8 @@
-#ifndef __defstoreh__
-  #include "defstore.h"
-#endif
-#ifndef __hitopcoreh__
-  #include "hitopcore.h"
-#endif
+#include "defstore.h"
 #ifndef __htmlh__
   #include "html.h"
 #endif
+
 #include <algorithm>
 
 static const string g_first("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_");
