@@ -2,10 +2,19 @@
 #define __dtutilsh__
 
 #include <string>
+#include <cctype>
+
+using namespace std;
+
+class ToUpper{
+public:
+  char operator()(char ch){return toupper(ch);}
+};
 
 extern bool isAbsolute(const string& s);
 extern string PathPart(string fname);
-extern string IToS(int Val);
+extern string IToS(int val);
+extern string FToS(float val);
 extern string FilePart(string fname);
 extern string escape(string data);
 extern string unescape(string urlEncoded) ;
