@@ -1,8 +1,7 @@
-#ifndef __gifh__
-  #include "gif.h"
-#endif
+#include "gif.h"
+#include <string.h>
 
-bool IsGIF(FILE *file){
+bool GIF::IsA(FILE *file){
   unsigned char header[8];
   if(fseek(file,0,SEEK_SET)!=0) return false;
   if(fread(header,1,6,file)!=6) return false;
@@ -10,7 +9,8 @@ bool IsGIF(FILE *file){
   return false;
 }
 
-bool GIFSize(FILE *file, int& height, int& width){
+bool GIF::GetSize(FILE *file, int& height, int& width){
+  if(!IsA(file)) return false;
   unsigned char hi,lo;
   if(fseek(file,6,SEEK_SET)!=0) return false;
   if(fread(&lo,1,1,file)!=1) return false;
