@@ -1,36 +1,35 @@
-#ifndef __hitopcommandsh__
-  #include "hitopcommands.h"
-#endif
-#ifndef __dtutilsh__
-  #include "dtutils.h"
-#endif
-#ifndef __hitopcoreh__
-  #include "hitopcore.h"
-#endif
+#include "hitopcommands.h"
 #ifndef __htmlh__
   #include "html.h"
 #endif
+#ifndef __parammaph__
+  #include "parammap.h"
+#endif
 #ifndef __errorh__
   #include "error.h"
 #endif
-#ifndef __defstoreh_
+#ifndef __imgstoreh__
+  #include "imgstore.h"
+#endif
+#ifndef __defstoreh__
   #include "defstore.h"
 #endif
 #ifndef __navh__
   #include "nav.h"
 #endif
-#ifndef __imgstoreh__
-  #include "imgstore.h"
+#ifndef __mathsh__
+  #include "maths.h"
 #endif
 #ifndef __pluginmgrh__
   #include "pluginmgr.h"
 #endif
-#ifndef __pluginh__
-  #include "plugin.h"
+#ifndef __polytypeh__
+  #include "polytype.h"
 #endif
+
 #include <fstream>
 #include <map>
-#include <time.h>
+#include <ctime>
 
 HTMLStream::iterator HitopCommands::IMPOSSIBLE(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
@@ -40,55 +39,63 @@ HTMLStream::iterator HitopCommands::IMPOSSIBLE(HTMLStream& stream,
 
 HTMLStream::iterator HitopCommands::IF(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
-  if(cur->m_myPair==NULL) Error(*cur,"@IF resulted from variable expansion");
+  if(cur->m_myPair==NULL){
+    if(cur->Token()==TokenMap::ELSEIF){
+      Error(*cur,"@ELSEIF resulted from variable expansion");
+    }else{
+      Error(*cur,"@IF resulted from variable expansion");
+    }
+  }
   string name,value,compare="EQ";
   HTMLStream::iterator next;
   bool eval=false;
-  if(!paramMap.Retrieve("NAME",name)) Error(*cur,e_NoName);
-  if(paramMap.Retrieve("COMPARE",compare)&&!g_isXML) transform(compare.begin(),compare.end(),compare.begin(),toupper);
-  if(paramMap.Retrieve("VALUE",value)){
-    string var;
-    if(!stream.m_curVars.Get(*cur,name,var)){
-      Error(*cur,e_UndefVar,name);
-    }else{
-      //string var=stream.m_curVars.Get(*cur,name);
-      if(compare=="EQ"){
-        eval=(var==value);
-      }else if(compare=="NE"){
-        eval=(var!=value);
-      }else if(compare=="LT"){
-	
-
-        eval=(var<value);
-      }else if(compare=="GT"){
-        eval=(var>value);
-      }else if(compare=="LE"){
-        eval=(var<=value);
-      }else if(compare=="GE"){
-        eval=(var>=value);
-      }else if(compare=="IN"){
-        eval=value.find(var)!=string::npos;
-      }else if(compare=="NI"){
-        eval=value.find(var)==string::npos;
-      }else if(compare=="EQINT"){
-        eval=(atoi(var.c_str())==atoi(value.c_str()));
-      }else if(compare=="NEINT"){
-        eval=(atoi(var.c_str())!=atoi(value.c_str()));
-      }else if(compare=="LTINT"){
-        eval=(atoi(var.c_str())<atoi(value.c_str()));
-      }else if(compare=="GTINT"){
-        eval=(atoi(var.c_str())>atoi(value.c_str()));
-      }else if(compare=="LEINT"){
-        eval=(atoi(var.c_str())<=atoi(value.c_str()));
-      }else if(compare=="GEINT"){
-        eval=(atoi(var.c_str())>=atoi(value.c_str()));
-      }else{
-        Error(*cur,e_BadCompare);
-      }
-    }
+  if(paramMap.Retrieve("EXPR",name)){
+    eval=PolyType(evaluate(name)).AsBool();
   }else{
-    eval=stream.m_curVars.IsDefName(name);
+    if(!paramMap.Retrieve("NAME",name)) Error(*cur,e_NoName);
+    if(paramMap.Retrieve("COMPARE",compare)&&!g_isXML) transform(compare.begin(),compare.end(),compare.begin(),ToUpper());
+    if(paramMap.Retrieve("VALUE",value)){
+      string var;
+      if(!stream.m_curVars.Get(*cur,name,var)){
+        Error(*cur,e_UndefVar,name);
+      }else{
+        if(compare=="EQ"){
+          eval=(var==value);
+        }else if(compare=="NE"){
+          eval=(var!=value);
+        }else if(compare=="LT"){
+          eval=(var<value);
+        }else if(compare=="GT"){
+          eval=(var>value);
+        }else if(compare=="LE"){
+          eval=(var<=value);
+        }else if(compare=="GE"){
+          eval=(var>=value);
+        }else if(compare=="IN"){
+          eval=value.find(var)!=string::npos;
+        }else if(compare=="NI"){
+          eval=value.find(var)==string::npos;
+        }else if(compare=="EQINT"){
+          eval=(atoi(var.c_str())==atoi(value.c_str()));
+        }else if(compare=="NEINT"){
+          eval=(atoi(var.c_str())!=atoi(value.c_str()));
+        }else if(compare=="LTINT"){
+          eval=(atoi(var.c_str())<atoi(value.c_str()));
+        }else if(compare=="GTINT"){
+          eval=(atoi(var.c_str())>atoi(value.c_str()));
+        }else if(compare=="LEINT"){
+          eval=(atoi(var.c_str())<=atoi(value.c_str()));
+        }else if(compare=="GEINT"){
+          eval=(atoi(var.c_str())>=atoi(value.c_str()));
+        }else{
+          Error(*cur,e_BadCompare);
+        }
+      }
+    }else{
+      eval=stream.m_curVars.IsDefName(name);
+    }
   }
+
   if(paramMap.Defined("NOT")) eval=!eval;
   if(cur->m_myElse==NULL){
     if(eval){
@@ -115,21 +122,40 @@ HTMLStream::iterator HitopCommands::IF(HTMLStream& stream,
       stream.m_stream.erase(cur);
     }else{
       next=cur->m_myElse;
-      ++next;
-      if(next==cur->m_myPair) ++next;
-      stream.m_stream.erase(cur->m_myPair);
-      ++cur->m_myElse;
-      stream.m_stream.erase(cur,cur->m_myElse);
+      if(next->Token()==TokenMap::ELSEIF){
+        stream.m_stream.erase(cur,cur->m_myElse);
+      }else{
+        ++next;
+        if(next==cur->m_myPair) ++next;
+        stream.m_stream.erase(cur->m_myPair);
+        ++cur->m_myElse;
+        stream.m_stream.erase(cur,cur->m_myElse);
+      }
     }
   }
   return next;
 }
 
+static bool FileToString(const string& fileName,string& content){
+  ifstream inputFile;
+  inputFile.open(fileName.c_str(),ios::in);
+  if(!inputFile.good()) return false;
+  char ch='\0';
+  while(!inputFile.eof()){
+    inputFile.get(ch);
+    if(!inputFile.eof()) content+=ch;
+  }
+  if(ch=='\n') content=content.substr(0,content.length()-1);
+  inputFile.close();
+  return true;
+}
+
 HTMLStream::iterator HitopCommands::SET(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
   Vars::Scope scope=Vars::Local;
   string scopeText;
   if(paramMap.Retrieve("SCOPE",scopeText)){
+    transform(scopeText.begin(),scopeText.end(),scopeText.begin(),ToUpper());
     if(scopeText=="LOCAL"){
       scope=Vars::Local;
     }else if(scopeText=="PARENT"){
@@ -147,6 +173,18 @@ HTMLStream::iterator HitopCommands::SET(HTMLStream& stream,
         if(!stream.m_curVars.Set(name,value,scope)){
           Error(*cur,e_BadVarName);
         }
+      }else if(paramMap.Retrieve("EXPR",value)){
+        value=evaluate(value);
+        if(!stream.m_curVars.Set(name,value,scope)){
+          Error(*cur,e_BadVarName);
+        }
+      }else if(paramMap.Retrieve("SRC",value)){
+        string text;
+        value=stream.m_ThisPath+value;
+        if(!FileToString(value,text)) Error(*cur,e_FileNotOpen,value);
+        if(!stream.m_curVars.Set(name,text,scope)){
+          Error(*cur,e_BadVarName);
+        }
       }else{
         if(!stream.m_curVars.Set(name,"",scope)){
           Error(*cur,e_BadVarName);
@@ -167,6 +205,7 @@ HTMLStream::iterator HitopCommands::UNSET(HTMLStream& stream,
   Vars::Scope scope=Vars::Local;
   string scopeText;
   if(paramMap.Retrieve("SCOPE",scopeText)){
+    transform(scopeText.begin(),scopeText.end(),scopeText.begin(),ToUpper());
     if(scopeText=="LOCAL"){
       scope=Vars::Local;
     }else if(scopeText=="PARENT"){
@@ -194,38 +233,62 @@ HTMLStream::iterator HitopCommands::UNSET(HTMLStream& stream,
 HTMLStream::iterator HitopCommands::GET(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
   string var,value,format;
-  if(!paramMap.Retrieve("NAME",var)) Error(*cur,e_NoName);
-  if(paramMap.Retrieve("FORMAT",format)) var+=":"+format;
-  if(!stream.m_curVars.Get(*cur,var,value)) Error(*cur,e_UndefVar,var);
-
-  *cur=value;
+  if(paramMap.Retrieve("EXPR",value)){
+    value=evaluate(value);
+  }else{
+    if(!paramMap.Retrieve("NAME",var)) Error(*cur,e_NoName);
+    if(paramMap.Retrieve("FORMAT",format)) var+=":"+format;
+    if(!stream.m_curVars.Get(*cur,var,value)) Error(*cur,e_UndefVar,var);
+  }
+  cur->ImposeText(value);
   ++cur;
   return cur;
 }
 
 HTMLStream::iterator HitopCommands::FILE(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
-  HTMLStream::iterator Next=cur;
-  ++Next;
-  string FName;
-  if(!paramMap.Retrieve("SRC",FName)) Error(*cur,e_NoSrc);
-  HTMLStream Insert(&stream);
-  Insert.m_debugLevel=stream.m_debugLevel;
-  Insert.m_FName=FName;
-  ifstream InFile;
-  Insert.m_ThisPath=PathPart(stream.m_ThisPath+FName);
-  InFile.open((stream.m_ThisPath+FName).c_str(),ios::in);
-  if(InFile.good()){
-    InFile>>Insert;
-    InFile.close();
-  }else{
-    Error(*cur,e_FileNotOpen);
+  HTMLStream::iterator next=cur;
+  ++next;
+  string filename,formatName;
+  if(!paramMap.Retrieve("SRC",filename)) Error(*cur,e_NoSrc);
+  enum FileFormat{Hitop,Verbatim,Safe};
+  FileFormat fileFormat=Hitop;
+  if(paramMap.Retrieve("FORMAT",formatName)){
+    if(!g_isXML) transform(formatName.begin(),formatName.end(),formatName.begin(),ToUpper());
+    if(formatName=="HITOP"){
+      fileFormat=Hitop;
+    }else if(formatName=="VERBATIM"){
+      fileFormat=Verbatim;
+    }else if(formatName=="SAFE"){
+      fileFormat=Safe;
+    }else{
+      Error(*cur,e_BadFormat2);
+    }
   }
-  Insert.Process();
-  stream.m_stream.splice(cur,Insert.m_stream);
-  //stream.m_stream.insert(cur,Insert.m_stream.begin(),Insert.m_stream.end());
-  stream.m_stream.erase(cur);
-  return Next;
+  if(fileFormat==Hitop || fileFormat==Safe){
+    HTMLStream insert(&stream);
+    insert.m_debugLevel=stream.m_debugLevel;
+    insert.m_FName=filename;
+    ifstream inputFile;
+    insert.m_ThisPath=PathPart(stream.m_ThisPath+filename);
+    inputFile.open((stream.m_ThisPath+filename).c_str(),ios::in);
+    if(inputFile.good()){
+      inputFile>>insert;
+      inputFile.close();
+    }else{
+      Error(*cur,e_FileNotOpen,stream.m_ThisPath+filename);
+    }
+    if(fileFormat==Hitop) insert.Process();
+    stream.m_stream.splice(cur,insert.m_stream);
+    stream.m_stream.erase(cur);
+  }else{
+    string text;
+    filename=stream.m_ThisPath+filename;
+    if(!FileToString(filename,text)) Error(*cur,e_FileNotOpen,filename);
+    cur->ImposeText(text);
+    cur->SetPreformat(true);
+  }
+  return next;
 }
 
 HTMLStream::iterator HitopCommands::UNDEF(HTMLStream& stream,
@@ -256,6 +319,7 @@ HTMLStream::iterator HitopCommands::HITOP(HTMLStream& stream,
   ++Next;
   paramMap.RetrieveNum("DEBUG",stream.m_debugLevel);
   paramMap.Retrieve("LIBS",PluginManager::s_loadpath);
+  paramMap.Retrieve("CONTENTTYPE",g_contentType);
   stream.m_stream.erase(cur);
   return Next;
 }
@@ -293,6 +357,15 @@ HTMLStream::iterator HitopCommands::DEF(HTMLStream& stream,
   return Next;
 }
 
+HTMLStream::iterator HitopCommands::COMMENT(HTMLStream& stream,
+  HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
+  HTMLStream::iterator Next=cur->m_myPair;
+  if(Next==NULL) Error(*cur,"@COMMENT resulted from variable expansion");
+  ++Next;
+  stream.m_stream.erase(cur,Next);
+  return Next;
+}
+
 HTMLStream::iterator HitopCommands::IMG(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
   string curPath,FName;
@@ -319,18 +392,65 @@ HTMLStream::iterator HitopCommands::IMG(HTMLStream& stream,
   return cur;
 }
 
+HTMLStream::iterator HitopCommands::ForEngine(HTMLStream& stream,
+  HTMLStream::iterator cur,const ParamMap& paramMap,TokenMap::Token
+  entryToken,int step){
+  int from=0,to=0;
+  if(!paramMap.RetrieveNum("FROM",from)) Error(*cur,e_NoFrom);
+  if(!paramMap.RetrieveNum("TO",to)) Error(*cur,e_NoTo);
+  if(step==0) Error(*cur,e_StepIsZero);
+  if(step<0) step=-step;
+  ParamMap newParam(paramMap);
+  if(from<to){
+    for(int i=from;i<=to;i+=step){
+      newParam["COUNT"]=IToS(i);
+      ProcCallByNum(stream,cur,newParam,entryToken);
+    }
+  }else{
+    for(int i=from;i>=to;i-=step){
+      newParam["COUNT"]=IToS(i);
+      ProcCallByNum(stream,cur,newParam,entryToken);
+    }
+  }
+  HTMLStream::iterator next=cur;
+  ++next;
+  stream.m_stream.erase(cur);
+  return next;
+}
+
+HTMLStream::iterator HitopCommands::FOR(HTMLStream& stream,
+  HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
+  TokenMap::Token entryToken=0;
+  int step=1;
+  if(!paramMap.RetrieveToken("ENTRY",entryToken)) Error(*cur,e_NoEntry);
+  if(!DEFStore::Exists(entryToken)) Error(*cur,e_UndefEntry);
+  paramMap.RetrieveNum("STEP",step);
+  return ForEngine(stream,cur,paramMap,entryToken,step);
+}
+
 HTMLStream::iterator HitopCommands::FOREACH(HTMLStream& stream,
   HTMLStream::iterator cur,const ParamMap& paramMap,const string& tag){
-  bool noData=true;
-  string entry;
-  if(!paramMap.Retrieve("ENTRY",entry)) Error(*cur,e_NoEntry);
-  TokenMap::Token token=TokenMap::NameToTokenNum(entry);
-  if(!DEFStore::Exists(token)) Error(*cur,e_UndefEntry);
-  string fname;
-  if(paramMap.Retrieve("SRC",fname)){
+  TokenMap::Token entryToken=0;
+  if(!paramMap.RetrieveToken("ENTRY",entryToken)) Error(*cur,e_NoEntry);
+  if(!DEFStore::Exists(entryToken)) Error(*cur,e_UndefEntry);
+  string src,data,prefix;
+  bool hasPREFIX=paramMap.Retrieve("PREFIX",prefix);
+  bool hasSRC=paramMap.Retrieve("SRC",src);
+  bool hasCONTENT=paramMap.Retrieve("CONTENT",data);
+  bool hasDATA=paramMap.Retrieve("DATA",data);
+  if(hasPREFIX){
+    map<string,string> results;
+    stream.m_curVars.Prefix(results,prefix);
+    ParamMap newParam(paramMap);
+    for(map<string,string>::const_iterator i=results.begin();i!=results.end();++i){
+      newParam["NAME"]=i->first;
+      newParam["VALUE"]=i->second;
+      ProcCallByNum(stream,cur,newParam,entryToken);
+    }
+  }else if(hasSRC){
     ifstream inFile;
-    inFile.open(fname.c_str(),ios::in);
-    if(!inFile.good()) Error(*cur,e_FileNotOpen,fname);
+    inFile.open(src.c_str(),ios::in);
+    if(!inFile.good()) Error(*cur,e_FileNotOpen,src);
     int count=1;
     ParamMap newParam(paramMap);
     string line;
@@ -339,18 +459,13 @@ HTMLStream::iterator HitopCommands::FOREACH(HTMLStream& stream,
       if(!inFile.eof()){
         newParam["CONTENT"]=EscapeSignificant(line);
         newParam["COUNT"]=IToS(count);
-        ProcCallByNum(stream,cur,newParam,token);
+        ProcCallByNum(stream,cur,newParam,entryToken);
         ++count;
       }
     }
     inFile.close();
-  }else{
-    string data;
-    if(paramMap.Retrieve("CONTENT",data)){
-      Warn(*cur,"CONTENT is deprecated in favour of DATA");
-      noData=false;
-    }
-    if(paramMap.Retrieve("DATA",data)) noData=false;
+  }else if(hasCONTENT||hasDATA){
+    if(hasCONTENT) Warn(*cur,"CONTENT is deprecated in favour of DATA");
     string sep;
     paramMap.Retrieve("SEP",sep);
     int from=1,to=65535;
@@ -359,24 +474,29 @@ HTMLStream::iterator HitopCommands::FOREACH(HTMLStream& stream,
     data+=sep;
     ParamMap newParam(paramMap);
     string current;
-    for(int count=1;(count<=to)&&((!data.empty())||noData);++count){
-      if(!noData){
-        int sepLength=sep.length();
-        if(sepLength>0){
-          int pos=data.find(sep);
-          current=data.substr(0,pos);
-          data=data.substr(pos+sepLength,-1);
-        }else{
-          current=data[0];
-          data.erase(0,1);
+    int sepLength=sep.length();
+    if(sepLength==0){
+      if(from<1) from=1;
+      if(to>(int)data.length()) to=data.length();
+      for(int count=from;count<=to;++count){
+        newParam["CONTENT"]=data[count-1];
+        newParam["COUNT"]=IToS(count);
+        ProcCallByNum(stream,cur,newParam,entryToken);
+      }
+    }else{
+      for(int count=1;(count<=to)&&(!data.empty());++count){
+        int pos=data.find(sep);
+        current=data.substr(0,pos);
+        data=data.substr(pos+sepLength,string::npos);
+        if(count>=from){
+          newParam["CONTENT"]=current;
+          newParam["COUNT"]=IToS(count);
+          ProcCallByNum(stream,cur,newParam,entryToken);
         }
       }
-      if((count>=from)&&(count<=to)){
-        newParam["CONTENT"]=current;
-        newParam["COUNT"]=IToS(count);
-        ProcCallByNum(stream,cur,newParam,token);
-      }
     }
+  }else{
+    return ForEngine(stream,cur,paramMap,entryToken,1);
   }
   HTMLStream::iterator next=cur;
   ++next;
@@ -395,7 +515,7 @@ HTMLStream::iterator HitopCommands::RAWHEADER(HTMLStream& stream,
   if(!paramMap.Retrieve("NAME",name)) Error(*cur,e_NoName);
   if(!paramMap.Retrieve("VALUE",value)) Error(*cur,e_NoValue);
   string temp;
-  if(!g_isXML) transform(name.begin(),name.end(),temp.begin(),toupper);
+  if(!g_isXML) transform(name.begin(),name.end(),temp.begin(),ToUpper());
   if(temp=="STATUS"){
     g_statusHeader=value;
   }else{
@@ -422,7 +542,7 @@ HTMLStream::iterator HitopCommands::COOKIE(HTMLStream& stream,
       time_t ttl = time(NULL)+atoi(maxage.c_str());
       struct tm *expiry= gmtime(&ttl);
       char buffer[80];
-      strftime(buffer,80,"%A, %d-%b-%Y %T GMT",expiry);    
+      strftime(buffer,80,"%A, %d-%b-%Y %H:%M:%S GMT",expiry);    
       expires=buffer;
     }
   }
@@ -480,7 +600,7 @@ HTMLStream::iterator HitopCommands::NAV(HTMLStream& stream,
   ++Next;
 
   string FormatStr;
-  if (paramMap.Retrieve("FORMAT",FormatStr)&&!g_isXML) transform(FormatStr.begin(),FormatStr.end(),FormatStr.begin(),toupper);
+  if (paramMap.Retrieve("FORMAT",FormatStr)&&!g_isXML) transform(FormatStr.begin(),FormatStr.end(),FormatStr.begin(),ToUpper());
   NavStore::OutputFormat format =NavStore::of_open;
 
   if (FormatStr == "FLAT") {
@@ -534,7 +654,7 @@ HTMLStream::iterator HitopCommands::NAV(HTMLStream& stream,
 HTMLStream::iterator HitopCommands::REQUIRES(HTMLStream& stream,HTMLStream::iterator cur,
   const ParamMap& paramMap,const string& tag) {
   string ns,src;
-  if (paramMap.Retrieve("NAMESPACE",ns)&&!g_isXML) transform(ns.begin(),ns.end(),ns.begin(),toupper);
+  if (paramMap.Retrieve("NAMESPACE",ns)&&!g_isXML) transform(ns.begin(),ns.end(),ns.begin(),ToUpper());
   if (!paramMap.Retrieve("PLUGIN",src)) Error(*cur,"PLUGIN must be defined for a plugin to be loaded");
   int minversion=0;
   paramMap.RetrieveNum("VERSION",minversion);
@@ -565,11 +685,15 @@ void HitopCommands::Init(){
     HTMLStream::RegisterCommand("NAV",&NAV);
     HTMLStream::RegisterCommand("SET",&SET);
   }
+  HTMLStream::RegisterCommand("@COMMENT",&COMMENT);
+  HTMLStream::RegisterCommand("/@COMMENT",&IMPOSSIBLE);
   HTMLStream::RegisterCommand("@DEBUG",&DEBUG);
   HTMLStream::RegisterCommand("@DEF",&DEF);
   HTMLStream::RegisterCommand("/@DEF",&IMPOSSIBLE);
   HTMLStream::RegisterCommand("@ELSE",&IMPOSSIBLE);
+  HTMLStream::RegisterCommand("@ELSEIF",&IF);
   HTMLStream::RegisterCommand("@FILE",&FILE);
+  HTMLStream::RegisterCommand("@FOR",&FOR);
   HTMLStream::RegisterCommand("@FOREACH",&FOREACH);
   HTMLStream::RegisterCommand("@GET",&GET);
   HTMLStream::RegisterCommand("@HITOP",&HITOP);
