@@ -4,6 +4,8 @@
 #include <map>
 #include <string>
 
+using namespace std;
+
 class tm;
 class HTMLStream;
 
@@ -12,6 +14,7 @@ extern const string g_dateISO;
 extern const string g_years;
 extern const bool g_isXML;
 extern const bool g_isHTML;
+extern string g_contentType;
 
 extern multimap<string,string> g_httpHeader;
 extern string g_statusHeader;
