@@ -1,15 +1,21 @@
 #ifndef __pluginh__
 #define __pluginh__
 
-#ifndef __dbmgrh__
-  #include "dbmgr.h"
-#endif
 #ifndef __stringproch__
   #include "stringproc.h"
 #endif
+#ifndef __htmlstreamh__
+  #include "htmlstream.h"
+#endif
+#ifndef __dbmgrh__
+  #include "dbmgr.h"
+#endif
+
 #include <string>
 #include <map>
 
+using namespace std;
+
 typedef ProcFunc CommandFunc;
 typedef StringProc::actionFunction FunctionFunc;
 
