@@ -4,13 +4,10 @@
 #ifndef __htmlstreamh__
   #include "../htmlstream.h"
 #endif
-
-#ifndef __htmlh__
-  #include "../html.h"
+#ifndef __pluginh__
+  #include "../plugin.h"
 #endif
 
-#include "../plugin.h"
-
 class dsoCalMod :public Plugin{
 public:
   dsoCalMod();
