@@ -1,18 +1,12 @@
-#ifndef __htmlstreamh__
-  #include "../htmlstream.h"
-#endif
-
-#ifndef __htmlh__
-  #include "../html.h"
-#endif
-
-#ifndef __stringproch__
-  #include "../stringproc.h"
-#endif
-
 #ifndef __pluginh__
   #include "../plugin.h"
 #endif
+#ifndef __errorh__
+  #include "../error.h"
+#endif
+#ifndef __polytypeh__
+  #include "../polytype.h"
+#endif
 
 #include <string>
 #include <vector>
@@ -22,7 +16,7 @@ public:
   dsoEcommerceMod();
   virtual void Init();
 private:
-  static void VALIDCARDNUM(const HTML& Cur,string& data,const vector<string>& params);
+  static void VALIDCARDNUM(__HITOPFUNC__);
 };
 
 static dsoEcommerceMod initmodule;
@@ -35,11 +29,11 @@ void dsoEcommerceMod::Init(){
   Plugin::RegisterFunction("VALIDCARDNUM",&dsoEcommerceMod::VALIDCARDNUM);
 }
 
-void dsoEcommerceMod::VALIDCARDNUM(const HTML& cur,string& data,const vector<string>& params){
+void dsoEcommerceMod::VALIDCARDNUM(__HITOPFUNC__){
   if(params.size()!=0) Error(cur,"VALIDCARDNUM",e_ParamNone);
   string::reverse_iterator pos;
   int mult=1,sum=0,prod,digits=0;
-  for(pos=data.rbegin();pos!=data.rend();++pos){
+  for(pos=data.AsString().rbegin();pos!=data.AsString().rend();++pos){
     if(isdigit(*pos)){
       ++digits;
       prod=(*pos-'0')*mult;
