@@ -1,6 +1,7 @@
 #include "png.h"
+#include <string.h>
 
-bool IsPNG(FILE *file){
+bool PNG::IsA(FILE *file){
   unsigned char header[10];
   if(fseek(file,0,SEEK_SET)!=0) return false;
   if(fread(header,1,8,file)!=8) return false;
@@ -8,7 +9,8 @@ bool IsPNG(FILE *file){
   return false;
 }
 
-bool PNGSize(FILE *file, int& height, int& width){
+bool PNG::GetSize(FILE *file, int& height, int& width){
+  if(!IsA(file)) return false;
   unsigned char length[4];
   if(fseek(file,8+4+4,SEEK_SET)!=0) return false;
   if(fread(&length,4,1,file)!=1) return false;
